SystemOrganization addCategory: #'Path-Contracts-Builder'!
SystemOrganization addCategory: #'Path-Contracts-Core'!
SystemOrganization addCategory: #'Path-Contracts-Evaluation'!
SystemOrganization addCategory: #'Path-Contracts-Exceptions'!

!Exception methodsFor: '*Path-Contracts' stamp: 'mp 3/15/2012 09:50:32.036'!
isContractViolation
	^ false! !

!BlockClosure methodsFor: '*Path-Contracts' stamp: 'ls 1/6/2012 15:13:01.696'!
forkWithContracts
	
	^ self forkWithContracts: SProject allProjectNames! !

!BlockClosure methodsFor: '*Path-Contracts' stamp: 'or 1/12/2012 17:30:10.984'!
forkWithContracts: activeGroups
	| process policy |
	process := self newProcess.
	policy := DCExecutionPolicy executionEnabled.
	policy activeGroups: activeGroups.
	process 
		contractExecutionPolicy: policy;
		name: 'Contract Execution: ', self outerContext asString;
		priority: Processor userBackgroundPriority.
	^ process resume! !

!BlockClosure methodsFor: '*Path-Contracts' stamp: 'ls 1/6/2012 15:11:43.441'!
forkWithoutContracts
	| process |
	process := self newProcess.
	process contractExecutionPolicy: DCExecutionPolicy executionDisabled.
	^ process resume! !

!BlockClosure methodsFor: '*Path-Contracts' stamp: 'ls 1/13/2012 13:01:34.932'!
newProcess
	"Answer a Process running the code in the receiver. The process is not 
	scheduled."
	| process |
	<primitive: 19> "Simulation guard"
	process := Process
		forContext: 
			[self value.
			Processor terminateActive] asContext
		priority: Processor activePriority.
	process contractExecutionPolicy: Processor activeProcess contractExecutionPolicy.
	^ process! !

!BlockClosure methodsFor: '*Path-Contracts' stamp: 'ls 1/13/2012 12:08:22.096'!
runWithContracts
	
	^ self runWithContracts: SProject allProjectNames! !

!BlockClosure methodsFor: '*Path-Contracts' stamp: 'ls 1/13/2012 12:09:43.729'!
runWithContracts: activeGroups
	| policy oldPolicy |
	oldPolicy := Processor activeProcess contractExecutionPolicy.
	policy := DCExecutionPolicy executionEnabled activeGroups: activeGroups.
	Processor activeProcess contractExecutionPolicy: policy.
	^ self ensure: [Processor activeProcess contractExecutionPolicy: oldPolicy]! !

!BlockClosure methodsFor: '*Path-Contracts' stamp: 'ls 1/13/2012 12:09:47.386'!
runWithoutContracts
	| oldPolicy |
	oldPolicy := Processor activeProcess contractExecutionPolicy.
	Processor activeProcess contractExecutionPolicy: DCExecutionPolicy executionDisabled.
	^ self ensure: [Processor activeProcess contractExecutionPolicy: oldPolicy]! !

MwMethodWrapper subclass: #DCContractWrapper
	instanceVariableNames: 'beforeMethod afterMethod result oldObject requiresOldObject preconditionGroups postconditionGroups'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Core'!

!DCContractWrapper methodsFor: 'accessing' stamp: 'or 11/30/2011 19:45:35.702'!
afterMethod
	^ afterMethod! !

!DCContractWrapper methodsFor: 'accessing' stamp: 'or 1/18/2012 16:56:12.964'!
afterMethod: aMethod
	"Groups are cached for performance reasons (accessing pragmas
	seems to be very time expensive) Same goes for old object."
	self
		postconditionGroups: (DCContract extractGroupsFrom: aMethod);
		requiresOldObject: (DCContract requiresOldObjectFor: aMethod).
	afterMethod := aMethod.! !

!DCContractWrapper methodsFor: 'accessing' stamp: 'or 11/23/2011 13:47:09.967'!
beforeMethod
	^ beforeMethod! !

!DCContractWrapper methodsFor: 'accessing' stamp: 'or 1/18/2012 16:54:53.801'!
beforeMethod: aMethod
	"Groups are cached for performance reasons (accessing pragmas
	seems to be very time expensive)"
	self preconditionGroups: (DCContract extractGroupsFrom: aMethod).
	beforeMethod := aMethod! !

!DCContractWrapper methodsFor: 'execution' stamp: 'ls 1/18/2012 17:40:04.198'!
containsActiveGroup: aGroup policy: policy
	aGroup isEmpty ifTrue: [^ policy activeGroups includes: #null].
	^  (aGroup anySatisfy: [:group | policy activeGroups includes: group])! !

!DCContractWrapper methodsFor: 'compiled method' stamp: 'or 1/31/2012 14:11:46.619'!
getSourceFor: selector in: class
	"Polymorphic with CompiledMethod"
	^ self clientMethod getSourceFor: selector in: class! !

!DCContractWrapper methodsFor: 'execution' stamp: 'or 1/18/2012 16:33:57.572'!
invariantGroups
	^ self wrappedClass organization invariantGroups! !

!DCContractWrapper methodsFor: 'execution' stamp: 'or 1/31/2012 14:15:23.587'!
invariantsWithReceiver: anObject policy: policy
	"Dynamic Contract: Invocation of preconditions for executed method"
	(policy invariantsActive and: 
	[self containsActiveGroup: self invariantGroups policy: policy]) ifTrue: 
		[anObject class organization invariantMethod 
			valueWithReceiver: anObject arguments: {}] ! !

!DCContractWrapper methodsFor: 'execution' stamp: 'ls 1/6/2012 13:58:06.379'!
isContractWrapper

	^ true! !

!DCContractWrapper methodsFor: 'accessing' stamp: 'ls 1/14/2012 22:45:01.38'!
oldObject

	^ oldObject! !

!DCContractWrapper methodsFor: 'accessing' stamp: 'ls 1/14/2012 22:44:56.78'!
oldObject: anObject

	oldObject := anObject! !

!DCContractWrapper methodsFor: 'accessing' stamp: 'ls 1/18/2012 17:35:44.002'!
postconditionGroups

	^ postconditionGroups! !

!DCContractWrapper methodsFor: 'accessing' stamp: 'ls 1/18/2012 17:35:37.369'!
postconditionGroups: anObject

	postconditionGroups := anObject! !

!DCContractWrapper methodsFor: 'execution' stamp: 'or 1/31/2012 14:15:42.001'!
postconditionsWithReceiver: anObject arguments: args policy: policy
	"Dynamic Contract: Invocation of postconditions for executed method"
	(policy postconditionsActive and: 
	[self containsActiveGroup: self postconditionGroups policy: policy]) ifTrue:
		[self afterMethod valueWithReceiver: anObject arguments: args]! !

!DCContractWrapper methodsFor: 'accessing' stamp: 'ls 1/18/2012 17:35:31.382'!
preconditionGroups

	^ preconditionGroups! !

!DCContractWrapper methodsFor: 'accessing' stamp: 'ls 1/18/2012 17:35:24.524'!
preconditionGroups: anObject

	preconditionGroups := anObject! !

!DCContractWrapper methodsFor: 'execution' stamp: 'or 1/31/2012 14:15:46.731'!
preconditionsWithReceiver: anObject arguments: args policy: policy
	"Dynamic Contract: Invocation of preconditions for executed method"
	(policy preconditionsActive and: 
	[self containsActiveGroup: self preconditionGroups policy: policy]) ifTrue: 
		[self beforeMethod valueWithReceiver: anObject arguments: args]! !

!DCContractWrapper methodsFor: 'accessing' stamp: 'or 1/31/2012 14:14:31.096'!
requiresOldObject
	"Lazy initialization should only take place if a compiled method sneaked into the 	system	which hasn't been compiled through tools with Dynamic Contract support"
	^ requiresOldObject ifNil: 
		[requiresOldObject := DCContract requiresOldObjectFor: self afterMethod]! !

!DCContractWrapper methodsFor: 'accessing' stamp: 'ls 1/17/2012 19:38:26.819'!
requiresOldObject: anObject

	requiresOldObject := anObject! !

!DCContractWrapper methodsFor: 'accessing' stamp: 'ls 12/6/2011 23:04:46.119'!
result

	^ result! !

!DCContractWrapper methodsFor: 'accessing' stamp: 'ls 12/6/2011 23:04:51.205'!
result: anObject

	result := anObject! !

!DCContractWrapper methodsFor: 'execution' stamp: 'or 1/31/2012 14:17:49.292'!
valueWithReceiver: anObject arguments: anArray
	"Dynamic Contracts use this method to enforce contracts on the actual method"
	| policy |
	policy := Processor activeProcess contractExecutionPolicy copy.
	policy ifNotActive: 
		[^ self clientMethod valueWithReceiver: anObject arguments: anArray].
	self requiresOldObject ifTrue: [self oldObject: anObject veryDeepCopy].	
	
	[self 
		invariantsWithReceiver: anObject policy: policy;
		preconditionsWithReceiver: anObject arguments: anArray policy: policy
	] runWithoutContracts.
	
	self result: (self clientMethod valueWithReceiver: anObject arguments: anArray).
	
	[self 
		postconditionsWithReceiver: anObject arguments: anArray policy: policy; 
		invariantsWithReceiver: anObject policy: policy
	] runWithoutContracts.
	
	^ self result! !

MwMethodWrapper subclass: #DCMutationWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Evaluation'!

!DCMutationWrapper commentStamp: 'or 1/31/2012 14:47:46.884' prior: 0!
A DCMutationWrapper is used to inject possible defects via the mutation engine.

Instance Variables
!

!DCMutationWrapper methodsFor: 'as yet unclassified' stamp: 'ls 1/5/2012 12:16:30.973'!
valueWithReceiver: anObject arguments: anArrayOfObjects

	^ anObject! !

Object subclass: #DCContract
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Core'!

!DCContract class methodsFor: 'accessing' stamp: 'or 1/31/2012 13:51:40.35'!
blacklistedTypes
	"No contracts will be generated that require an object to be kind of a type
	listed below "
	^ { Object. SOMUniverse}! !

!DCContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 17:36:57.619'!
check: anObject and: aClass IfPossible: aBlock
	anObject ifNil: [^ self].
	(anObject class includesBehavior: aClass) ifFalse: [^ self].
	((anObject class includesBehavior: String) and: [aClass = Collection]) ifTrue: [^ self].
	aBlock value.! !

!DCContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 17:40:38.851'!
checkSpellingOf: aString
	^ (SpellingInterface new
		setIgnoredWords: #('foo');
		checkSpellingOf: aString startingAt: 1) second = 0! !

!DCContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 16:58:20.921'!
compare: anObject with: anotherObject
	^ anotherObject isSymbol
			ifTrue: [anObject perform: anotherObject]
			ifFalse: [anObject = anotherObject]! !

!DCContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 16:56:08.378'!
convertToComparableValue: anObject type: aClass
	^ (aClass includesBehavior: DateAndTime)
			ifTrue: [DateAndTime fromString: anObject]
			ifFalse: [anObject].! !

!DCContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 17:06:16.759'!
expect: anObject ofType: aClass is: anotherObject
	self check: anObject and: aClass IfPossible: [
		(self compare: anObject with: anotherObject)
			ifFalse: [self signalContractViolation]].! !

!DCContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 17:11:19.621'!
expect: anObject ofType: aClass isBetween: min and: max
	| minValue maxValue |
	self check: anObject and: aClass IfPossible: [
		minValue := self convertToComparableValue: min type: aClass.
		maxValue :=  self convertToComparableValue: max type: aClass.
		(anObject between: minValue and: maxValue)
			ifFalse: [self signalContractViolation]].! !

!DCContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 17:10:32.906'!
expect: anObject ofType: aClass isNot: anotherObject
	self check: anObject and: aClass IfPossible: [
		(self compare: anObject with: anotherObject)
			ifTrue: [self signalContractViolation]].! !

!DCContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 17:31:22.193'!
expect: anObject ofType: aClass property: aSymbol includes: aCollection
	self check: anObject and: aClass IfPossible: [
		(aCollection includes: (self getPropertyFrom: anObject with: aSymbol))
			ifFalse: [self signalContractViolation]].! !

!DCContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 17:22:57.539'!
expect: anObject ofType: aClass property: aSymbol isBetween: min and: max
	| minValue maxValue |
	self check: anObject and: aClass IfPossible: [
		minValue := self convertToComparableValue: min type: aClass.
		maxValue :=  self convertToComparableValue: max type: aClass.	
		((self getPropertyFrom: anObject with: aSymbol) 
			between: minValue and: maxValue)
				ifFalse: [self signalContractViolation]].! !

!DCContract class methodsFor: 'type contracts' stamp: 'mp 3/15/2012 12:30:27.655'!
expect: aParameter toBeKindOf: aType
	| context |
	aParameter ifNil: [^ self].
	(aParameter isKindOf: aType) 
		ifFalse: [context := self identifyWrapperContext.
				DCTypeContractViolation 
					signalOn: (self getClientMethod: context)
					kindOf: (self getContractType: context)
					given: aParameter class
					expected: aType]
	! !

!DCContract class methodsFor: 'type contracts' stamp: 'ls 1/18/2012 17:52:38.707'!
expectAllElementsOf: aCollection toBeKindOf: aType

	aCollection ifNil: [^ self].
	aCollection do: [:each | self expect: each toBeKindOf: aType]! !

!DCContract class methodsFor: 'type contracts' stamp: 'ls 1/18/2012 17:52:28.305'!
expectAllKeysOf: aDictionary toBeKindOf: aType

	aDictionary ifNil: [^ self].
	aDictionary keysDo: [:each | self expect: each toBeKindOf: aType]! !

!DCContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 17:40:07.341'!
expectCorrectSpellingOf: aString
	self check: aString and: String IfPossible: [
		(self checkSpellingOf: aString)
			ifFalse: [self signalContractViolation]].! !

!DCContract class methodsFor: 'type contracts' stamp: 'ls 1/18/2012 17:49:28.965'!
expectElementsOf: aCollection toBeKindOf: aType

	aCollection ifNil: [^ self].
	aCollection do: [:each | self expect: each toBeKindOf: aType. ^ self]! !

!DCContract class methodsFor: 'type contracts' stamp: 'ls 1/18/2012 17:53:06.939'!
expectKeysOf: aDictionary toBeKindOf: aType

	aDictionary ifNil: [^ self].
	aDictionary keysDo: [:each | self expect: each toBeKindOf: aType. ^ self]! !

!DCContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 17:09:59.715'!
expectNotNil: anObject
	anObject ifNil: [self signalContractViolation]! !

!DCContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 17:32:53.814'!
expectSize: anObject ofType: aClass isLessThan: aNumber
	self check: anObject and: aClass IfPossible: [
		(anObject size <= aNumber)
			ifFalse: [self signalContractViolation]].! !

!DCContract class methodsFor: 'Contract enforcement' stamp: 'or 1/31/2012 13:56:28.022'!
extractGroupsFrom: aContractMethod

	| pragma |
	aContractMethod ifNil: [^ nil].
	"This works since each contract method contains exactly one of our pragmas"
	pragma := (aContractMethod pragmas select: [:p | 
		Object contractPragmas includes: p keyword]) first.
	^ pragma arguments first ifNil: [Set new]! !

!DCContract class methodsFor: 'exceptions-helper' stamp: 'mp 3/15/2012 11:20:33.648'!
getClientMethod: aContext
	^ aContext receiver clientMethod asMethodReference! !

!DCContract class methodsFor: 'exceptions-helper' stamp: 'mp 3/15/2012 11:26:44.919'!
getContractType: aContext
	| wrapperSelector |
	wrapperSelector := aContext method selector.
	(wrapperSelector beginsWith: #preconditions)
		ifTrue: [^ #precondition].
	(wrapperSelector beginsWith: #invariants)
		ifTrue: [^ #invariant].
	(wrapperSelector beginsWith: #postconditions)
		ifTrue: [^ #postcondition].				! !

!DCContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 17:22:41.818'!
getPropertyFrom: anObject with: aSymbol
	^ aSymbol substrings 
		inject: anObject 
		into: [:result :each | result perform: each asSymbol]! !

!DCContract class methodsFor: 'exceptions-helper' stamp: 'mp 3/15/2012 11:15:22.079'!
identifyWrapperContext 
	| context |
	context := thisContext.
	[context notNil and: [context receiver isContractWrapper not]] 
		whileTrue: [context := context sender].
	context ifNil: [self halt "should not be triggered but if we can debug it!!"].
	^ context! !

!DCContract class methodsFor: 'Contract enforcement' stamp: 'or 1/18/2012 16:51:40.346'!
oldObject
	"You should only call this method from within a postcondition"
	^ thisContext sender sender sender receiver oldObject! !

!DCContract class methodsFor: 'Contract enforcement' stamp: 'ls 1/17/2012 19:37:23.562'!
requiresOldObjectFor: aPostconditionMethod

	^ aPostconditionMethod sendsSelector: #oldObject! !

!DCContract class methodsFor: 'Contract enforcement' stamp: 'ls 1/20/2012 14:13:16.984'!
result
	"You should only call this method from within a postcondition"
	^ thisContext sender sender sender receiver result! !

!DCContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 17:06:40.356'!
signalContractViolation
	| context |
	context := self identifyWrapperContext.
	DCContractViolation
		signalOn: (self getClientMethod: context) 
		kindOf: (self getContractType: context).! !

Object subclass: #DCContractBuilder
	instanceVariableNames: 'selector arguments groups typeInfo valueRangeInfo sources compiledMethod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Builder'!

!DCContractBuilder methodsFor: 'configuration' stamp: 'ls 12/27/2011 16:38:29.271'!
addGroup: aSymbol

	self groups addIfNotPresent: aSymbol asSymbol! !

!DCContractBuilder methodsFor: 'templates' stamp: 'ls 12/28/2011 17:05:44.913'!
argumentTemplate
	
	^ 'DCContract expect: <ARGUMENT> toBeKindOf: <TYPE>.'! !

!DCContractBuilder methodsFor: 'accessing' stamp: 'ls 12/27/2011 13:19:40.507'!
arguments

	^ arguments ifNil: [arguments := OrderedCollection new]! !

!DCContractBuilder methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:12:59.192'!
arguments: anObject

	arguments := anObject! !

!DCContractBuilder methodsFor: 'building' stamp: 'ls 12/27/2011 00:51:13.315'!
build
	
	self
		buildSelector;
		buildPragma;
		buildContracts! !

!DCContractBuilder methodsFor: 'building' stamp: 'ls 12/27/2011 01:02:18.82'!
buildAndStoreIn: aClass
	
	self	 
		build;
		storeIn: aClass! !

!DCContractBuilder methodsFor: 'building' stamp: 'mp 3/20/2012 11:48:18.453'!
buildContracts
	self typeInfo 
		ifNotNil: [self 
					printSourceLine: '"Type Contracts"';
					buildTypeContracts].
	self valueRangeInfo
		ifNotNil: [self 
					printSourceLine: '"Value Range Contracts"';
					buildValueRangeContracts].! !

!DCContractBuilder methodsFor: 'building' stamp: 'or 1/31/2012 13:59:51.993'!
buildPragma

	| pragma groups |
	groups := WriteStream on: String new.
	self groups do: [:group | group storeOn: groups] separatedBy: [groups space].
	pragma := self pragmaTemplate 
		copyReplaceTokens: '<GROUPS>' with: groups contents.
	self printSourceLine: pragma! !

!DCContractBuilder methodsFor: 'building' stamp: 'ls 12/27/2011 12:57:10.622'!
buildSelector

	| selectorWithArgs |
	selectorWithArgs := WriteStream on: String new.
	self selector numArgs = 0 ifTrue: [
		self sources nextPutAll: self selector keywords first.
		^ self].
	
	(1 to: self selector numArgs) do: [:nr |
		selectorWithArgs
			nextPutAll: (self selector keywords at: nr); space;
			nextPutAll: (arguments at: nr); space].
	self sources nextPutAll: selectorWithArgs contents! !

!DCContractBuilder methodsFor: 'building' stamp: 'mp 3/20/2012 11:41:59.771'!
buildTypeContracts
	self subclassResponsibility.! !

!DCContractBuilder methodsFor: 'building' stamp: 'mp 3/20/2012 11:42:12.451'!
buildValueRangeContracts
	self subclassResponsibility.! !

!DCContractBuilder methodsFor: 'templates' stamp: 'ls 12/27/2011 18:31:10.167'!
collectionTemplate
	
	^ 'DCContract expectElementsOf: <COLLECTION> toBeKindOf: <TYPE>.'! !

!DCContractBuilder methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:13:05.137'!
compiledMethod

	^ compiledMethod! !

!DCContractBuilder methodsFor: 'accessing' stamp: 'ls 12/27/2011 01:04:52.437'!
compiledMethod: anObject

	compiledMethod := anObject! !

!DCContractBuilder methodsFor: 'templates' stamp: 'ls 12/27/2011 18:51:06.128'!
dictionaryTemplate
	
	^ 'DCContract expectKeysOf: <DICTIONARY> toBeKindOf: <TYPE>.'! !

!DCContractBuilder methodsFor: 'accessing' stamp: 'ls 12/27/2011 13:19:28.101'!
groups

	^ groups ifNil: [groups := OrderedCollection new]! !

!DCContractBuilder methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:13:14.375'!
groups: anObject

	groups := anObject! !

!DCContractBuilder methodsFor: 'templates' stamp: 'ls 12/27/2011 00:20:44.572'!
pragmaTemplate

	^ self subclassResponsibility! !

!DCContractBuilder methodsFor: 'templates' stamp: 'or 1/31/2012 13:57:38.852'!
printArgumentContractFor: anArgumentName typed: aType
	
	| contract |
	(anArgumentName isNil or: [aType isNil]) ifTrue: [^ self].
	(DCContract blacklistedTypes includes: aType) ifTrue: [^ self].
	contract := self argumentTemplate 
		copyReplaceTokens: '<ARGUMENT>' with: anArgumentName.
	contract := contract copyReplaceTokens: '<TYPE>' with: aType asString.
	self printSourceLine: contract! !

!DCContractBuilder methodsFor: 'templates' stamp: 'or 1/31/2012 13:58:51.692'!
printCollectionContractFor: aCollectionName typed: aType
	
	| contract |
	(aCollectionName isNil or: [aType isNil]) ifTrue: [^ self].
	(DCContract blacklistedTypes includes: aType) ifTrue: [^ self].
	contract := self collectionTemplate 
		copyReplaceTokens: '<COLLECTION>' with: aCollectionName.
	contract := contract copyReplaceTokens: '<TYPE>' with: aType asString.
	self printSourceLine: contract! !

!DCContractBuilder methodsFor: 'templates' stamp: 'mp 3/20/2012 10:33:31.276'!
printContainerContractFor: aContainerName typed: aTHContainerType
	
	| type |
	self flag: #mpe. "Bug: THContainer can be nested infinitely but in most cases depth of 2 is sufficient"
	aTHContainerType typeOfContainer ifNotNil: [
		(aTHContainerType typeOfContainer isKindOf: THContainerType)
			ifTrue: [type := aTHContainerType typeOfContainer typeOfContainer]
			ifFalse: [type := aTHContainerType typeOfContainer].
		self printArgumentContractFor: aContainerName typed: type].
	
	aTHContainerType typeOfKeys ifNotNil: [
		(aTHContainerType typeOfKeys isKindOf: THContainerType) 
			ifTrue: [type := aTHContainerType typeOfKeys typeOfContainer]
			ifFalse: [type := aTHContainerType typeOfKeys].
		self printDictionaryContractFor: aContainerName typed: type].
	
	aTHContainerType typeOfElements ifNotNil: [
		(aTHContainerType typeOfElements isKindOf: THContainerType) 
			ifTrue: [type := aTHContainerType typeOfElements typeOfContainer]
			ifFalse: [type := aTHContainerType typeOfElements].
		self printCollectionContractFor: aContainerName typed: type].! !

!DCContractBuilder methodsFor: 'templates' stamp: 'mp 3/20/2012 16:51:39.945'!
printContractsFor: anArgumentName valueRanges: aCollectionOfRanges

	aCollectionOfRanges do: [:each | each printContractFor: anArgumentName with: self].		
! !

!DCContractBuilder methodsFor: 'templates' stamp: 'or 1/31/2012 13:59:20.949'!
printDictionaryContractFor: aDictionaryName typed: aType
	
	| contract |
	(aDictionaryName isNil or: [aType isNil]) ifTrue: [^ self].
	(DCContract blacklistedTypes includes: aType) ifTrue: [^ self].
	contract := self dictionaryTemplate 
		copyReplaceTokens: '<DICTIONARY>' with: aDictionaryName.
	contract := contract copyReplaceTokens: '<TYPE>' with: aType asString.
	self printSourceLine: contract! !

!DCContractBuilder methodsFor: 'templates' stamp: 'ls 1/5/2012 12:17:01.809'!
printSourceLine: aString

	self sources
		nextPut: Character cr;
		nextPut: Character tab;
		nextPutAll: aString! !

!DCContractBuilder methodsFor: 'accessing' stamp: 'ls 12/26/2011 23:39:19.669'!
selector

	^ selector! !

!DCContractBuilder methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:13:19.861'!
selector: anObject

	selector := anObject! !

!DCContractBuilder methodsFor: 'configuration' stamp: 'ls 12/28/2011 17:14:45.308'!
setSelector: aSymbol argumentNames: aCollection

	self 
		selector: aSymbol;
		arguments: aCollection	! !

!DCContractBuilder methodsFor: 'accessing' stamp: 'ls 12/27/2011 00:34:24.34'!
sources

	^ sources ifNil: [sources := WriteStream on: String new]! !

!DCContractBuilder methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:13:24.488'!
sources: anObject

	sources := anObject! !

!DCContractBuilder methodsFor: 'building' stamp: 'ls 12/27/2011 12:55:23.391'!
storeIn: aClass

	| methodNode |
	methodNode := (aClass 
		compile:  self sources contents 
		classified: nil 
		notifying: nil 
		trailer: aClass defaultMethodTrailer 
		ifFail: [^ nil]).
	methodNode method 
		putSource: self sources contents 
		fromParseNode: methodNode node inFile: 2
		withPreamble: [:f | f cr; nextPut: $!!; nextChunkPut: 'Contract Method'; cr].
	self compiledMethod: methodNode method! !

!DCContractBuilder methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:13:34.042'!
typeInfo

	^ typeInfo! !

!DCContractBuilder methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:13:29.901'!
typeInfo: anObject

	typeInfo := anObject! !

!DCContractBuilder methodsFor: 'accessing' stamp: 'mp 3/20/2012 11:09:04.035'!
valueRangeInfo
	"Answer the value of valueRangeInfo"

	^ valueRangeInfo! !

!DCContractBuilder methodsFor: 'accessing' stamp: 'mp 3/20/2012 11:09:04.047'!
valueRangeInfo: anObject
	"Set the value of valueRangeInfo"

	valueRangeInfo := anObject! !

DCContractBuilder subclass: #DCInvariantBuilder
	instanceVariableNames: 'instVars'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Builder'!

!DCInvariantBuilder methodsFor: 'configuration' stamp: 'ls 12/27/2011 00:19:08.594'!
addInstVar: aName

	self instVars addIfNotPresent: aName! !

!DCInvariantBuilder methodsFor: 'configuration' stamp: 'ls 12/27/2011 00:19:19.544'!
addInstVars: aCollection

	aCollection do: [:instVar | self addInstVar: instVar]! !

!DCInvariantBuilder methodsFor: 'building' stamp: 'mp 3/20/2012 11:42:25.832'!
buildTypeContracts

	| memberType |
	self instVars withIndexDo: [:instVarName :i |
		memberType := self typeInfo at: i.
		(memberType isKindOf: THContainerType)
			ifTrue: [self printContainerContractFor: instVarName typed: memberType]
			ifFalse: [self printArgumentContractFor: instVarName typed: memberType]]! !

!DCInvariantBuilder methodsFor: 'building' stamp: 'mp 3/20/2012 13:45:45.62'!
buildValueRangeContracts
	| memberRanges |
	self instVars withIndexDo: [:instVarName :i |
		memberRanges := self valueRangeInfo at: i.
		memberRanges ifNotNil: [
			self printContractsFor: instVarName 
				valueRanges: memberRanges asOrderedCollection]]! !

!DCInvariantBuilder methodsFor: 'accessing' stamp: 'ls 12/27/2011 13:18:47.697'!
instVars
	"Answer the value of instVars"

	^ instVars ifNil: [instVars := OrderedCollection new]! !

!DCInvariantBuilder methodsFor: 'accessing' stamp: 'ls 12/27/2011 00:19:31.928'!
instVars: anObject
	"Set the value of instVars"

	instVars := anObject! !

!DCInvariantBuilder methodsFor: 'accessing' stamp: 'ls 12/27/2011 00:20:28.141'!
pragmaTemplate

	^ '<invariants: #(<GROUPS>)>'! !

DCContractBuilder subclass: #DCPostconditionBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Builder'!

!DCPostconditionBuilder methodsFor: 'building' stamp: 'mp 3/20/2012 11:42:45.363'!
buildTypeContracts
	
	(self typeInfo isKindOf: THContainerType) 
		ifTrue: [self printContainerContractFor: '(DCContract result)' typed: self typeInfo]
		ifFalse: [self printResultContractTyped: self typeInfo]! !

!DCPostconditionBuilder methodsFor: 'building' stamp: 'mp 3/20/2012 16:41:33.031'!
buildValueRangeContracts
	self printContractsFor: 'DCContract result' 
		valueRanges: self valueRangeInfo asOrderedCollection! !

!DCPostconditionBuilder methodsFor: 'templates' stamp: 'ls 12/27/2011 00:24:26.155'!
pragmaTemplate

	^ '<postconditions: #(<GROUPS>)>'! !

!DCPostconditionBuilder methodsFor: 'templates' stamp: 'ls 1/11/2012 22:48:57.074'!
printResultContractTyped: aType
	
	| contract |
	aType ifNil: [^ self].
	(DCContract blacklistedTypes includes: aType) ifTrue: [^ self].
	contract := self resultTemplate copyReplaceTokens: '<TYPE>' with: aType asString.
	self printSourceLine: contract! !

!DCPostconditionBuilder methodsFor: 'templates' stamp: 'ls 12/28/2011 17:00:40.203'!
resultTemplate
	
	^ 'DCContract expect: (DCContract result) toBeKindOf: <TYPE>.'! !

DCContractBuilder subclass: #DCPreconditionBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Builder'!

!DCPreconditionBuilder methodsFor: 'building' stamp: 'mp 3/20/2012 13:34:58.767'!
buildTypeContracts
	| argumentType |
	self arguments withIndexDo: [:arg :i | 
		argumentType := self typeInfo at: i.
		(argumentType isKindOf: THContainerType)
			ifTrue: [self printContainerContractFor: arg typed: argumentType]
			ifFalse: [self printArgumentContractFor: arg typed: argumentType]]! !

!DCPreconditionBuilder methodsFor: 'building' stamp: 'mp 3/20/2012 13:44:43.143'!
buildValueRangeContracts
	| argumentRange |
	self arguments withIndexDo: [:arg :i | 
		argumentRange := self valueRangeInfo at: i.
		argumentRange ifNotNil: [
			self printContractsFor: arg 
				valueRanges: argumentRange asOrderedCollection]].! !

!DCPreconditionBuilder methodsFor: 'templates' stamp: 'ls 12/27/2011 00:23:41.123'!
pragmaTemplate

	^ '<preconditions: #(<GROUPS>)>'! !

Object subclass: #DCContractInjector
	instanceVariableNames: 'methodsInScope typesBucket valueRangeBucket groupName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Builder'!

!DCContractInjector class methodsFor: 'instance creation' stamp: 'mp 3/20/2012 10:59:07.453'!
covering: aSProject
	"returns a DCContractInjector instance for the given classes
	which will generate pre- and postconditions as well as invariants"
	| instance knownMethods |
	instance := self new.
	knownMethods := Dictionary new.
	aSProject allClasses do: [:class | knownMethods at: class put: class selectors].
	instance 
		methodsInScope: knownMethods;
		typesBucket: aSProject typesBucket;
		valueRangeBucket: aSProject valueRangeBucket;
		groupName: aSProject projectName.
		
	^ instance
! !

!DCContractInjector class methodsFor: 'instance creation' stamp: 'ls 12/22/2011 16:29:52.15'!
coveringCurrentSProject
	
	^ self covering: SProject current! !

!DCContractInjector methodsFor: 'public api' stamp: 'ls 12/23/2011 18:31:40.361'!
apply
	
	'Injecting Contracts'
		displayProgressAt: Display center
		from: 0 to: self classesInScope size
		during: [:bar | 1 to: self classesInScope size do: [:i |
				bar value: (self classesInScope at: i) printString.
				self injectContractsIn: (self classesInScope at: i).
				bar value: i]]! !

!DCContractInjector methodsFor: 'accessing' stamp: 'ls 11/27/2011 12:36:21.881'!
classesInScope

	^ self methodsInScope keys! !

!DCContractInjector methodsFor: 'contract creation' stamp: 'mp 3/20/2012 11:33:51.575'!
generateInvariantsIn: aClass

	| builder selector |
	builder := DCInvariantBuilder new.
	self typesBucket 
		ifNotNil: [builder typeInfo: (self typesBucket memberTypeMapFor: aClass)].
	self valueRangeBucket
		ifNotNil: [builder valueRangeInfo: (self valueRangeBucket 
												memberRangesFor: aClass)].
	selector := aClass asString copyReplaceAll: ' ' with: '' asTokens: false.
	builder
		setSelector: selector argumentNames: {};
		addGroup: self groupName;
		addInstVars: aClass allInstVarNames;
		buildAndStoreIn: aClass.
	^ builder compiledMethod
! !

!DCContractInjector methodsFor: 'contract creation' stamp: 'mp 3/20/2012 11:24:15.098'!
generatePostconditionsOn: aSymbol in: aClass
	
	| argumentNames builder |
	argumentNames := (aClass methodDict at: aSymbol) methodNode arguments collect: 		[:a | a name].
	builder := DCPostconditionBuilder new. 
	self typesBucket 
		ifNotNil: [builder typeInfo: (self typesBucket returnTypeFor: 
										(aClass methodDict at: aSymbol))].
	self valueRangeBucket
		ifNotNil: [builder valueRangeInfo: (self valueRangeBucket returnRangeFor: 
											(aClass methodDict at: aSymbol))].
	builder
		setSelector: aSymbol argumentNames: argumentNames;
		addGroup: self groupName;
		buildAndStoreIn: aClass.
	^ builder compiledMethod
	! !

!DCContractInjector methodsFor: 'contract creation' stamp: 'mp 3/20/2012 11:32:50.087'!
generatePreconditionsOn: aSymbol in: aClass
	
	| argumentNames builder |
	argumentNames := (aClass methodDict at: aSymbol) methodNode arguments collect: 		[:a | a name].
	builder := DCPreconditionBuilder new. 
	self typesBucket 
		ifNotNil: [builder typeInfo: (self typesBucket parameterTypesFor: 
												(aClass methodDict at: aSymbol))].
	self valueRangeBucket
		ifNotNil: [builder valueRangeInfo: (self valueRangeBucket parameterRangesFor: 												(aClass methodDict at: aSymbol))].
	builder 
		setSelector: aSymbol argumentNames: argumentNames;
		addGroup: self groupName;
		buildAndStoreIn: aClass.
	^ builder compiledMethod! !

!DCContractInjector methodsFor: 'accessing' stamp: 'ls 12/27/2011 18:41:34.524'!
groupName

	^ groupName! !

!DCContractInjector methodsFor: 'accessing' stamp: 'ls 12/27/2011 18:41:39.012'!
groupName: anObject

	groupName := anObject! !

!DCContractInjector methodsFor: 'contract creation' stamp: 'ls 1/18/2012 12:50:36.066'!
injectContractsIn: aClass
	
	| selectors |
	aClass organization invariantMethod: (self generateInvariantsIn: aClass).
	selectors := self methodsInScope at: aClass.
	selectors do: [:selector | self injectContractsOn: selector in: aClass].
	"Class side"
	aClass class organization invariantMethod: (self generateInvariantsIn: aClass class).
	aClass class selectors do: [:selector | self injectContractsOn: selector in: aClass class]! !

!DCContractInjector methodsFor: 'contract creation' stamp: 'or 1/18/2012 16:55:13.418'!
injectContractsOn: aSelector in: aClass
	
	| methodWrapper afterMethod |
	"First, make sure that no nested contracts are generated"
	self removeContractsOn: aSelector in: aClass.
	methodWrapper := DCContractWrapper on: aSelector inClass: aClass.
	afterMethod := (self generatePostconditionsOn: aSelector in: aClass).
	methodWrapper 
		beforeMethod: (self generatePreconditionsOn: aSelector in: aClass);
		afterMethod: afterMethod.
	methodWrapper install! !

!DCContractInjector methodsFor: 'accessing' stamp: 'ls 12/23/2011 18:26:08.69'!
methodsInScope

	^ methodsInScope! !

!DCContractInjector methodsFor: 'accessing' stamp: 'ls 12/23/2011 18:26:16.258'!
methodsInScope: anObject

	methodsInScope := anObject! !

!DCContractInjector methodsFor: 'contract removal' stamp: 'or 1/31/2012 14:08:11.324'!
removeContractsIn: aClass

	| selectors |
	aClass organization invariantMethod: nil.
	selectors := self methodsInScope at: aClass.
	selectors do: [:selector | self removeContractsOn: selector in: aClass].
	
	aClass class organization invariantMethod: nil.
	aClass class selectors do: [:selector | 
		self removeContractsOn: selector in: aClass class]! !

!DCContractInjector methodsFor: 'contract removal' stamp: 'ls 1/6/2012 13:59:13.859'!
removeContractsOn: aSelector in: aClass

	| method |
	method := aClass methodDictionary at: aSelector.
	method isContractWrapper ifTrue: [method uninstall]! !

!DCContractInjector methodsFor: 'public api' stamp: 'ls 12/25/2011 17:41:13.815'!
revert
	'Removing Contracts'
		displayProgressAt: Display center
		from: 0 to: self classesInScope size
		during: [:bar | 1 to: self classesInScope size do: [:i |
				bar value: (self classesInScope at: i) printString.
				self removeContractsIn: (self classesInScope at: i).
				bar value: i]]! !

!DCContractInjector methodsFor: 'accessing' stamp: 'ls 12/23/2011 18:12:31.907'!
typesBucket

	^ typesBucket! !

!DCContractInjector methodsFor: 'accessing' stamp: 'ls 12/23/2011 18:12:25.612'!
typesBucket: anObject

	typesBucket := anObject! !

!DCContractInjector methodsFor: 'accessing' stamp: 'mp 3/20/2012 10:58:40.477'!
valueRangeBucket
	"Answer the value of valueRangeBucket"

	^ valueRangeBucket! !

!DCContractInjector methodsFor: 'accessing' stamp: 'mp 3/20/2012 10:58:40.488'!
valueRangeBucket: anObject
	"Set the value of valueRangeBucket"

	valueRangeBucket := anObject! !

Object subclass: #DCEvaluator
	instanceVariableNames: 'coveredProject pathMap coveredMethods testResults'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Evaluation'!

!DCEvaluator class methodsFor: 'instance creation' stamp: 'or 1/16/2012 14:27:17.417'!
on: anSProject
	| instance |
	instance := self new.
	instance 
		coveredProject: anSProject.
	^instance! !

!DCEvaluator class methodsFor: 'instance creation' stamp: 'or 1/16/2012 14:13:31.518'!
onCurrentSProject
	
	^ self on: SProject current! !

!DCEvaluator class methodsFor: 'accessing' stamp: 'or 1/31/2012 14:33:57.274'!
resultHeader
	^ (Dictionary newFromPairs: {
		#tag. 'Contracts enabled'. 
		#methodReference. 'Mutated method'. 
		#testFails. 'Test failures'. 
		#testErrors. 'Test errors'. 
		#contractFails. 'Failing contracts'})! !

!DCEvaluator methodsFor: 'csv file' stamp: 'ls 1/5/2012 13:51:56.377'!
convertCollectionToCSV: aCollection  on: aStream

	aCollection withIndexDo: [:row :index |
		aStream nextPutAll: index asString.
		aStream nextPut: $,.
		row do: [:column | 
			aStream nextPutAll: column asString.
			aStream nextPut: $,.].
		aStream cr; flush.].! !

!DCEvaluator methodsFor: 'accessing' stamp: 'ls 1/15/2012 14:49:46.697'!
coveredMethods
	"Answer the value of coveredMethods"

	^ coveredMethods! !

!DCEvaluator methodsFor: 'accessing' stamp: 'ls 1/15/2012 14:49:46.711'!
coveredMethods: anObject
	"Set the value of coveredMethods"

	coveredMethods := anObject! !

!DCEvaluator methodsFor: 'accessing' stamp: 'ls 1/5/2012 13:28:29.534'!
coveredProject

	^ coveredProject! !

!DCEvaluator methodsFor: 'accessing' stamp: 'ls 1/5/2012 13:28:34.768'!
coveredProject: anObject

	coveredProject := anObject! !

!DCEvaluator methodsFor: 'public api' stamp: 'ls 1/5/2012 14:46:49.616'!
do: aBlock withCurrentProject: aProjectName
	| previousCurrentProject result |
	[previousCurrentProject := SProject current.
	SProject setCurrent: aProjectName.
	result := aBlock value]
		ensure: [SProject setCurrentProject: previousCurrentProject].
	^ result! !

!DCEvaluator methodsFor: 'public api' stamp: 'ls 1/15/2012 14:55:46.882'!
evaluate

	^ self evaluateFromEntryPoint: (self methodsCoveredByTests first)! !

!DCEvaluator methodsFor: 'public api' stamp: 'mp 3/15/2012 09:49:39.892'!
evaluateFromEntryPoint: aMethodReference
	"Start iterating at aMethodReference and mutate all covered methods"
	| mutationEngine contractFailures |
	self testResults: (OrderedCollection with: self class resultHeader).
	self do: 
		[mutationEngine := DCMutationEngine on: self methodsCoveredByTests.
		mutationEngine mutationsDo: [:methodReference |
			"Transcript cr; show: 'Mutated ', methodReference asString; cr."
			[self pathMap runAllTests] runWithoutContracts.
			self 
				evaluateTestStateOf: self pathMap testRunner 
				with: #nocontracts 
				forMethodReference: methodReference.
			contractFailures := 0.
			[[self pathMap runAllTests] runWithContracts: 
				self coveredProject projectName] 
			on: DCContractViolation  do: [:exception | 
				"Transcript show: 'Contract: ', exception messageText; cr."
				contractFailures := contractFailures + 1. 
				exception return: false].
			self 
				evaluateTestStateOf: self pathMap testRunner 
				withTag: #contracts 
				forMethodReference: methodReference 
				andContractFails: contractFailures
				
		] entryPoint: aMethodReference.
		] withCurrentProject: self coveredProject projectName.
	self storeCollection: self testResults asCSVFile: 'Evaluation ', TimeStamp now, '.csv'.
	self showTestResults.! !

!DCEvaluator methodsFor: 'public api' stamp: 'or 1/31/2012 14:31:44.1'!
evaluateTestStateOf: aTestRunner with: aTag forMethodReference: methodReference
	^ self 
		evaluateTestStateOf: aTestRunner 
		withTag: aTag 
		forMethodReference: methodReference 
		andContractFails: 0! !

!DCEvaluator methodsFor: 'public api' stamp: 'or 1/31/2012 14:32:39.033'!
evaluateTestStateOf: aTestRunner withTag: aTag forMethodReference: methodReference andContractFails: anInteger

	| failures errors results |
	failures := aTestRunner failedList size.
	errors := aTestRunner errorList size.
	results := Dictionary newFromPairs: 
		{#tag. aTag. 
		#methodReference. (methodReference actualClass name, '>>', methodReference methodSymbol) asSymbol. 
		#testFails. failures. 
		#testErrors. errors. 
		#contractFails. anInteger}.
	self testResults add: results.! !

!DCEvaluator methodsFor: 'accessing' stamp: 'ls 1/5/2012 13:31:33.725'!
initializePathMap

	| testRunner |
	testRunner := ToolBuilder build: SPathMapTestRunner new.
	self pathMap: testRunner model pathMap.
	self pathMap globalOptions
		switchShowQuickAndAccessorMethods;
		switchShowMethodCategories;
		switchVisualizeIntermediateSteps.! !

!DCEvaluator methodsFor: 'accessing' stamp: 'ls 1/15/2012 14:57:56.046'!
methodsCoveredByTests

	^ self coveredMethods ifNil: 
		[self initializePathMap.
		self pathMap runAllTestsWith: #testQuality.
		self coveredMethods: (self pathMap tracer getAllCoveredMethods keys 
		reject: [:each | ((each methodSymbol beginsWith: 'test') 
						or: [each compiledMethod isQuick])
						or: [each actualClass inheritsFrom: TestCase]]).
		^ self coveredMethods]! !

!DCEvaluator methodsFor: 'accessing' stamp: 'ls 1/6/2012 15:23:45.383'!
pathMap

	^ pathMap! !

!DCEvaluator methodsFor: 'accessing' stamp: 'ls 1/6/2012 15:23:50.17'!
pathMap: anObject

	pathMap := anObject! !

!DCEvaluator methodsFor: 'accessing' stamp: 'or 1/31/2012 14:27:44.549'!
showTestResults
	| contracts noContracts contractFailed noDifference contractOnly noFail |
	
	contractFailed := 0.
	noDifference := 0.
	contractOnly := 0.
	noFail := 0.
	
	Transcript show: '--- Results (see also logFile) ---'; cr.
	noContracts := self testResults select: [:item | (item at: #tag) = #nocontracts].
	contracts := self testResults select: [:item | (item at: #tag) = #contracts].
	
	noContracts do: [:item | | withContract | 
		withContract := contracts detect: [:c | 
			((c at: #methodReference) = (item at: #methodReference))].
		"A Contract catched an anomaly"		
		((withContract at: #contractFails) > 0) 
		ifTrue:
			[(((item at: #testFails) = 0) and: [(item at: #testErrors) = 0]) 
			"The tests did not notice the anomaly"
			ifTrue: [contractOnly := contractOnly + 1]
			"The anomaly was catched before a test failed"
			ifFalse: [((withContract at: #testFails) = 0) 
				ifTrue: [contractFailed := contractFailed + 1]]]
		ifFalse: [(((withContract at: #testFails) = 0) and: 
			[(withContract at: #testErrors) = 0]) 
			"no one discovered the anomaly"
			ifTrue: [noFail := noFail + 1]
			ifFalse: [noDifference := noDifference + 1]]].
				
	Transcript show: 'Mutated ', ((self testResults size - 1) / 2) asString, ' methods.'; cr.
	Transcript show: 'No difference: ', noDifference asString, 
		' Contract failed: ', contractFailed, 
		' Only contract failed: ', contractOnly, 
		' Anomaly unnoticed: ', noFail; cr! !

!DCEvaluator methodsFor: 'csv file' stamp: 'ls 1/5/2012 13:51:42.98'!
storeCollection: aCollection asCSVFile: aFileName
	| file |
	[file := FileDirectory default forceNewFileNamed: aFileName.
	 self convertCollectionToCSV: aCollection  on: file]
	ensure: [file close].! !

!DCEvaluator methodsFor: 'accessing' stamp: 'or 1/16/2012 13:52:18.13'!
testResults
	"Answer the value of testResults"

	^ testResults! !

!DCEvaluator methodsFor: 'accessing' stamp: 'or 1/16/2012 13:52:18.191'!
testResults: anObject
	"Set the value of testResults"

	testResults := anObject! !

Object subclass: #DCExecutionPolicy
	instanceVariableNames: 'activeGroups preconditionsActive postconditionsActive invariantsActive'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Core'!

DCExecutionPolicy class
	instanceVariableNames: 'defaultPolicy'!

DCExecutionPolicy class
	instanceVariableNames: 'defaultPolicy'!

!DCExecutionPolicy class methodsFor: 'instance creation' stamp: 'ls 1/31/2012 17:35:11.942'!
default
	^ defaultPolicy ifNil: [defaultPolicy := DCExecutionPolicy executionDisabled]! !

!DCExecutionPolicy class methodsFor: 'instance creation' stamp: 'ls 1/18/2012 11:49:26.422'!
executionDisabled
	| instance |
	instance := self new.
	instance
		activeGroups: {};
		preconditionsActive: false;
		postconditionsActive: false;
		invariantsActive: false.
	^ instance! !

!DCExecutionPolicy class methodsFor: 'instance creation' stamp: 'ls 1/18/2012 11:49:44.272'!
executionEnabled
	| instance |
	instance := self new.
	instance 
		activeGroups: SProject allProjectNames;
		preconditionsActive: true;
		postconditionsActive: true;
		invariantsActive: true.
	^ instance! !

!DCExecutionPolicy class methodsFor: 'public api' stamp: 'or 1/31/2012 14:54:49.04'!
nuke
	"remove policies of all processes so that they will get the new default policy"
	| processList |
	Smalltalk garbageCollectMost.
	processList := Process allSubInstances reject: [:each | each isTerminated].
	processList := WeakArray withAll: processList.
	processList do: [:each | each contractExecutionPolicy: nil].! !

!DCExecutionPolicy methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:07:49.928'!
activeGroups

	^ activeGroups! !

!DCExecutionPolicy methodsFor: 'accessing' stamp: 'or 1/13/2012 14:33:59.346'!
activeGroups: aGroupOrCollection

	aGroupOrCollection isSymbol
		ifTrue: [activeGroups := (Set with: aGroupOrCollection)]
		ifFalse: [activeGroups := aGroupOrCollection]
! !

!DCExecutionPolicy methodsFor: 'public api' stamp: 'ls 12/25/2011 18:49:07.367'!
ifNotActive: aBlock
	
	self isNotActive ifTrue: [^ aBlock value]! !

!DCExecutionPolicy methodsFor: 'accessing' stamp: 'ls 1/18/2012 11:48:35.084'!
invariantsActive
	"Answer the value of invariantsActive"

	^ invariantsActive! !

!DCExecutionPolicy methodsFor: 'accessing' stamp: 'ls 1/18/2012 11:48:35.133'!
invariantsActive: anObject
	"Set the value of invariantsActive"

	invariantsActive := anObject! !

!DCExecutionPolicy methodsFor: 'public api' stamp: 'ls 12/25/2011 18:29:59.661'!
isActive
	
	^ self invariantsActive or: [self preconditionsActive] or: [self postconditionsActive]! !

!DCExecutionPolicy methodsFor: 'public api' stamp: 'ls 12/25/2011 18:46:56.668'!
isNotActive
	
	^ self isActive not! !

!DCExecutionPolicy methodsFor: 'accessing' stamp: 'ls 1/18/2012 11:48:34.986'!
postconditionsActive
	"Answer the value of postconditionsActive"

	^ postconditionsActive! !

!DCExecutionPolicy methodsFor: 'accessing' stamp: 'ls 1/18/2012 11:48:35.034'!
postconditionsActive: anObject
	"Set the value of postconditionsActive"

	postconditionsActive := anObject! !

!DCExecutionPolicy methodsFor: 'accessing' stamp: 'ls 1/18/2012 11:48:34.878'!
preconditionsActive
	"Answer the value of preconditionsActive"

	^ preconditionsActive! !

!DCExecutionPolicy methodsFor: 'accessing' stamp: 'ls 1/18/2012 11:48:34.934'!
preconditionsActive: anObject
	"Set the value of preconditionsActive"

	preconditionsActive := anObject! !

Object subclass: #DCMutationEngine
	instanceVariableNames: 'coveredMethods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Evaluation'!

!DCMutationEngine class methodsFor: 'instance creation' stamp: 'ls 1/5/2012 13:09:39.365'!
on: aMethodReferenceCollection

	^ self new coveredMethods: aMethodReferenceCollection! !

!DCMutationEngine class methodsFor: 'public api' stamp: 'ls 1/5/2012 13:03:47.176'!
undoMutationsGlobally
	
	"First, do a garbage collection so no iteration takes place over no
	longer referenced DCMutationWrapper instances"
	Smalltalk garbageCollect.
	DCMutationWrapper allInstancesDo: [:instance |
		instance uninstall].! !

!DCMutationEngine methodsFor: 'checking' stamp: 'or 1/31/2012 14:43:24.773'!
blacklistedMethods
	"Methods listed below may lead to an image freeze if mutated"
	^ { 
		MethodReference class: SOMMethodParser2 selector: #countBracket.
		MethodReference class: SOMClassParser2 selector: #classInstanceSwitch.
		MethodReference class: SOMClassParser2 selector: #foreignParseMethod.
		MethodReference class: SOMMethodParser2 selector: #unaryMsg.
		MethodReference class: SOMMethodParser2 selector: #binaryMsg.
		MethodReference class: SOMMethodParser2 selector: #assignment.
		MethodReference class: SOMMethodParser2 selector: #binaryChar.
		MethodReference class: SOMMethodParser2 selector: #method.
		MethodReference class: SOMArray selector: #at:.
		MethodReference class: SOMArray selector: #add:.
		MethodReference class: SOMFrame selector: #at:put:.
		MethodReference class: SOMInterpreter selector: #popFrame.
		MethodReference class: SOMInterpreter selector: #somFrame:.
		MethodReference class: SOMClassGenerationContext selector: #addFields:.
		MethodReference class: SOMFrame selector: #pop.
		MethodReference class: SOMFrame selector: #stackElementAtIndex:.
		MethodReference class: SOMInteger class selector: #maxInt.
		MethodReference class: SOMMethodGenerationContext selector: #addLocalIfAbsent:.
		MethodReference class: SOMMethodGenerationContext selector: #outer:.
		MethodReference class: SOMParser2 selector: #comment.
		MethodReference class: SOMVariableLookup selector: #isArgument:.
		MethodReference class: SOMVariableLookup selector: #lookupArgument.
		MethodReference class: SOMVariableLookup selector: #lookupField.
		MethodReference class: SOMVariableLookup selector: #lookupLocal.
		MethodReference class: SOMVariableLookup selector: #isLocal:.
		MethodReference class: SOMVariableLookup selector: #isField:.
		MethodReference class: SOMBlock selector: #cacheEvaluationMethodWithParameters:.
		MethodReference class: SOMBlock selector: #somBlockMethod:.
	}! !

!DCMutationEngine methodsFor: 'accessing' stamp: 'ls 1/5/2012 12:19:44.904'!
coveredMethods

	^ coveredMethods! !

!DCMutationEngine methodsFor: 'accessing' stamp: 'ls 1/5/2012 12:19:48.577'!
coveredMethods: anObject

	coveredMethods := anObject! !

!DCMutationEngine methodsFor: 'checking' stamp: 'ls 1/15/2012 18:00:49.502'!
isReturningSelf: aMethodReference
	| method |
	method := aMethodReference compiledMethod.
	"^ method isReturnSelf or: [method scanFor: 16r78]"
	^ method isReturnSelf! !

!DCMutationEngine methodsFor: 'public api' stamp: 'ls 1/15/2012 14:07:59.874'!
mutationsDo: aBlock
	
	^ self mutationsDo: aBlock entryPoint: self coveredMethods first! !

!DCMutationEngine methodsFor: 'public api' stamp: 'or 1/31/2012 14:44:11.027'!
mutationsDo: aBlock entryPoint: aMethodReference
	| entryPointReached |
	entryPointReached := false.
	"Iterate over all covered methods and mutate the current one 
	before executing aBlock"
	self coveredMethods do: [:reference | | wrapper |
		entryPointReached := entryPointReached or: [reference = aMethodReference].
		(entryPointReached and: [self shouldMutate: reference]) ifTrue: 
			[wrapper := DCMutationWrapper 
				on: reference methodSymbol 
				inClass: reference actualClass.
			wrapper installClosestToMethod.
			[aBlock value: reference] ensure: [wrapper uninstall]]]! !

!DCMutationEngine methodsFor: 'checking' stamp: 'or 1/31/2012 14:44:57.679'!
shouldMutate: aMethodReference

	^ (self isReturningSelf: aMethodReference) not and: 
	  [(self blacklistedMethods includes: aMethodReference) not]! !

!Object class methodsFor: '*Path-Contracts' stamp: 'ls 12/28/2011 17:07:38.414'!
contractPragmas
	<pragmas: #instance>
	^ #(#preconditions:  #postconditions: #invariants:)! !

!Object methodsFor: '*Path-Contracts' stamp: 'ls 1/6/2012 13:57:20.542'!
isContractWrapper
	
	^ false! !

Halt subclass: #DCContractViolation
	instanceVariableNames: 'methodReference kindOf'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Exceptions'!

!DCContractViolation commentStamp: 'or 1/31/2012 14:00:45.398' prior: 0!
A DCContractFailure is raised if a contract fails.

Instance Variables
!

!DCContractViolation class methodsFor: 'instance creation' stamp: 'mp 3/15/2012 12:21:16.646'!
signalOn: aMethodReference
	^ self signalOn: aMethodReference kindOf: #unknown.! !

!DCContractViolation class methodsFor: 'instance creation' stamp: 'mp 3/15/2012 12:21:09.931'!
signalOn: aMethodReference kindOf: aSymbol
	^ self new
		messageText: 'Contract violation';
		methodReference: aMethodReference;
		kindOf: aSymbol;
		signal! !

!DCContractViolation methodsFor: 'comparing' stamp: 'mp 3/21/2012 15:57:14.037'!
< other
	^ false! !

!DCContractViolation methodsFor: 'comparing' stamp: 'mp 3/15/2012 14:19:53.704'!
= other
	self class = other class
		ifFalse: [^ false].
	^ (self methodReference = other methodReference and: [self kindOf = other kindOf])! !

!DCContractViolation methodsFor: 'printing' stamp: 'mp 3/15/2012 12:20:59.719'!
description
	^ self messageText, ' (', self kindOf asString, ') in ', self methodReference asString! !

!DCContractViolation methodsFor: 'comparing' stamp: 'mp 3/15/2012 14:22:57.59'!
hash
	^ self methodReference hash bitXor: self kindOf hash! !

!DCContractViolation methodsFor: 'testing' stamp: 'mp 3/15/2012 09:49:51.836'!
isContractViolation
	^ true! !

!DCContractViolation methodsFor: 'testing' stamp: 'mp 3/15/2012 12:31:48.772'!
isTypeContractViolation
	^ false! !

!DCContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 12:20:49.46'!
kindOf
	"Answer the value of kindOf"

	^ kindOf! !

!DCContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 12:20:49.473'!
kindOf: anObject
	"Set the value of kindOf"

	kindOf := anObject! !

!DCContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 09:51:58.039'!
methodReference
	"Answer the value of methodReference"

	^ methodReference! !

!DCContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 09:51:58.051'!
methodReference: anObject
	"Set the value of methodReference"

	methodReference := anObject! !

!DCContractViolation methodsFor: 'printing' stamp: 'mp 3/30/2012 11:19:17.605'!
shortDescription
	"required for labels in PathMap and PathFinder - see also SPMViolationMorph"
	^ '(', self shortKindOf, ')'! !

!DCContractViolation methodsFor: 'printing' stamp: 'mp 3/30/2012 11:11:50.143'!
shortKindOf
	^ self kindOf caseOf: {
		[#precondition] -> ['pre'].
		[#invariant] -> ['inv'].
		[#postcondition] -> ['post'].
	}.! !

DCContractViolation subclass: #DCTypeContractViolation
	instanceVariableNames: 'givenType expectedType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Exceptions'!

!DCTypeContractViolation class methodsFor: 'instance creation' stamp: 'mp 3/15/2012 12:31:24.944'!
signalOn: aMethodRef kindOf: aSymbol given: aClassGi expected: aClassEx
	^ self new
		messageText: 'Type violation';
		methodReference: aMethodRef;
		kindOf: aSymbol;
		givenType: aClassGi;
		expectedType: aClassEx;
		signal! !

!DCTypeContractViolation methodsFor: 'comparing' stamp: 'mp 3/15/2012 14:25:29.677'!
= other
	self class = other class
		ifFalse: [^ false].
	^ self methodReference = other methodReference 
		and: [self kindOf = other kindOf]
		and: [self givenType = other givenType]
		and: [self expectedType = other expectedType]! !

!DCTypeContractViolation methodsFor: 'printing' stamp: 'mp 3/15/2012 17:56:09.083'!
description
	^ self messageText, 
	' (Given: ', self givenType asString,
	' Expected: ', self expectedType asString, ') ',
	self kindOf asString, ' fails in ',
	self methodReference asString! !

!DCTypeContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 12:28:04.62'!
expectedType
	"Answer the value of expectedType"

	^ expectedType! !

!DCTypeContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 12:28:04.631'!
expectedType: anObject
	"Set the value of expectedType"

	expectedType := anObject! !

!DCTypeContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 12:28:04.596'!
givenType
	"Answer the value of givenType"

	^ givenType! !

!DCTypeContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 12:28:04.608'!
givenType: anObject
	"Set the value of givenType"

	givenType := anObject! !

!DCTypeContractViolation methodsFor: 'comparing' stamp: 'mp 3/15/2012 14:26:39.993'!
hash
	^ (super hash bitXor: self givenType hash) bitXor: self expectedType hash! !

!DCTypeContractViolation methodsFor: 'testing' stamp: 'mp 3/15/2012 12:31:54.322'!
isTypeContractViolation
	^ true! !

!DCTypeContractViolation methodsFor: 'printing' stamp: 'mp 3/30/2012 11:19:33.962'!
shortDescription
	^ self givenType asString,
		' !!= ', self expectedType asString,
		' ', super shortDescription! !
